@@ -19,7 +19,7 @@
 These functions provide shorthand functional interfaces for actions such
 as getting a :class:`~dxpy.bindings.DXDataObject` handler from an ID or
 `link
-<https://wiki.dnanexus.com/API-Specification-v1.0.0/Details-and-Links#Linking>`_,
+<https://documentation.dnanexus.com/developer/api/data-object-lifecycle/details-and-links#linking>`_,
 or creating a link from a handler. In addition, there are functions for
 performing simple actions with an ID or link as input without creating a
 full object handler.
@@ -34,26 +34,44 @@ from . import __dict__ as all_bindings
 from ..exceptions import DXError
 from ..compat import basestring
 
-def dxlink(object_id, project_id=None):
+def dxlink(object_id, project_id=None, field=None):
     '''
     :param object_id: Object ID or the object handler itself
     :type object_id: string or :class:`~dxpy.bindings.DXDataObject`
     :param project_id: A project ID, if creating a cross-project DXLink
     :type project_id: string
+    :param field: A field name, if creating a job-based object reference
+    :type field: string
+    :returns: A dict formatted as a symbolic DNAnexus object reference
+    :rtype: dict
 
-    Creates a DXLink (a dict formatted as a symbolic DNAnexus object
-    reference) to the specified object.  Returns *object_id* if it
-    appears to be a DXLink already.
+    Creates a DXLink to the specified object.
+
+    If `object_id` is already a link, it is returned without modification.
+
+    If `object_id is a `~dxpy.bindings.DXDataObject`, the object ID is
+    retrieved via its `get_id()` method.
+
+    If `field` is not `None`, `object_id` is expected to be of class 'job'
+    and the link created is a Job Based Object Reference (JBOR), which is
+    of the form::
+
+        {'$dnanexus_link': {'job': object_id, 'field': field}}
+
+    If `field` is `None` and `project_id` is not `None`, the link created
+    is a project-specific link of the form::
+
+        {'$dnanexus_link': {'project': project_id, 'id': object_id}}
     '''
+    if is_dxlink(object_id):
+        return object_id
     if isinstance(object_id, DXDataObject):
         object_id = object_id.get_id()
-    if isinstance(object_id, dict):
-        if '$dnanexus_link' in object_id:
-            # In this case, dxlink was called on something that
-            # already looks like a link
-            return object_id
-    if project_id is None:
+    if not any((project_id, field)):
         return {'$dnanexus_link': object_id}
+    elif field:
+        dxpy.verify_string_dxid(object_id, "job")
+        return {'$dnanexus_link': {'job': object_id, 'field': field}}
     else:
         return {'$dnanexus_link': {'project': project_id, 'id': object_id}}
 
@@ -72,36 +90,46 @@ def is_dxlink(x):
     if isinstance(link, basestring):
         return True
     elif isinstance(link, dict):
-        return ('id' in link or 'job' in link)
-    else:
-        return False
+        return any(key in link for key in ('id', 'job'))
+    return False
 
 def get_dxlink_ids(link):
     '''
     :param link: A DNAnexus link
     :type link: dict
-    :returns: Object ID, Project ID (or :const:`None` if no project specified in the link)
+    :returns: (Object ID, Project ID) if the link is to a data object (or :const:`None`
+        if no project specified in the link), or (Job ID, Field) if the link is
+        a job-based object reference (JBOR).
     :rtype: tuple
 
-    Returns the object and project IDs stored in the given DNAnexus
-    link.
+    Get the object ID and detail from a link. There are three types of links:
+
+    * Simple link of the form ``{"$dnanexus_link": "file-XXXX"}`` returns
+      ``("file-XXXX", None)``.
+    * Data object link of the form ``{"$dnanexus_link': {"id": "file-XXXX",
+      "project": "project-XXXX"}}`` returns ``("file-XXXX", "project-XXXX")``.
+    * Job-based object reference (JBOR) of the form ``{"$dnanexus_link":
+      {"job": "job-XXXX", "field": "foo"}}`` returns ``("job-XXXX", "foo")``.
     '''
-    if isinstance(link['$dnanexus_link'], dict):
+    if not is_dxlink(link):
+        raise DXError('Invalid link: %r' % link)
+    if isinstance(link['$dnanexus_link'], basestring):
+        return link['$dnanexus_link'], None
+    elif 'id' in link['$dnanexus_link']:
         return link['$dnanexus_link']['id'], link['$dnanexus_link'].get('project')
     else:
-        return link['$dnanexus_link'], None
+        return link['$dnanexus_link']['job'], link['$dnanexus_link']['field']
 
-def _guess_link_target_type(link):
-    if is_dxlink(link):
-        # Guaranteed by is_dxlink that one of the following will work
-        if isinstance(link['$dnanexus_link'], basestring):
-            link = link['$dnanexus_link']
-        else:
-            link = link['$dnanexus_link']['id']
-    class_name, _id = link.split("-", 1)
-    class_name = 'DX'+class_name.capitalize()
-    if class_name == 'DXGtable':
-        class_name = 'DXGTable'
+def _guess_link_target_type(id_or_link):
+    # Get the object ID if the input is a link
+    object_id = get_dxlink_ids(id_or_link)[0] if is_dxlink(id_or_link) else id_or_link
+    class_name = 'DX' + object_id.split("-", 1)[0].capitalize()
+    if class_name not in all_bindings:
+        class_name = {
+            'DXGlobalworkflow': 'DXGlobalWorkflow'
+        }.get(class_name)
+    if class_name not in all_bindings:
+        raise DXError("Invalid class name: %s", class_name)
     cls = all_bindings[class_name]
     return cls
 
@@ -111,7 +139,7 @@ def get_handler(id_or_link, project=None):
     :type id_or_link: string or dict
     :param project: String project ID to use as the context if the the object is a data object
     :type project: string
-    :rtype: :class:`~dxpy.bindings.DXObject`
+    :rtype: :class:`~dxpy.bindings.DXObject`, :class:`~dxpy.bindings.DXApp`, or :class:`~dxpy.bindings.DXGlobalWorkflow`
 
     Parses a string or DXLink dict. Creates and returns an object handler for it.
 
@@ -124,7 +152,7 @@ def get_handler(id_or_link, project=None):
     except Exception as e:
         raise DXError("Could not parse link {}: {}".format(id_or_link, e))
 
-    if cls == dxpy.DXApp:
+    if cls in [dxpy.DXApp, dxpy.DXGlobalWorkflow]:
         # This special case should translate identifiers of the form
         # "app-name" or "app-name/version_or_tag" to the appropriate
         # arguments
@@ -132,10 +160,11 @@ def get_handler(id_or_link, project=None):
             return cls(id_or_link)
         else:
             slash_pos = id_or_link.find('/')
+            dash_pos = id_or_link.find('-')
             if slash_pos == -1:
-                return cls(name=id_or_link[4:])
+                return cls(name=id_or_link[dash_pos+1:])
             else:
-                return cls(name=id_or_link[4:slash_pos],
+                return cls(name=id_or_link[dash_pos+1:slash_pos],
                            alias=id_or_link[slash_pos + 1:])
     elif project is None or cls in [dxpy.DXJob, dxpy.DXAnalysis, dxpy.DXProject, dxpy.DXContainer]:
         # This case is important for the handlers which do not
@@ -147,15 +176,29 @@ def get_handler(id_or_link, project=None):
 def describe(id_or_link, **kwargs):
     '''
     :param id_or_link: String containing an object ID or dict containing a DXLink,
-                       or a list of object ID's or dict's containing a DXLink.
+                       or a list of object IDs or dicts containing a DXLink.
 
     Given an object ID, calls :meth:`~dxpy.bindings.DXDataObject.describe` on the object.
 
     Example::
 
         describe("file-1234")
+
+    Given a list of object IDs, calls :meth:`~dxpy.api.system_describe_data_objects`.
+
+    Example::
+
+        describe(["file-1234", "workflow-5678"])
+
+    Note: If id_or_link is a list and **kwargs contains a "fields" parameter, these
+    fields will be returned in the response for each data object in addition to the
+    fields included by default. Additionally, describe options can be provided for
+    each data object class in the "classDescribeOptions" kwargs argument. See
+    https://documentation.dnanexus.com/developer/api/system-methods#api-method-system-describedataobjects
+    for input parameters used with the multiple object describe method.
     '''
     # If this is a list, extract the ids.
+    # TODO: modify the procedure to use project ID when possible
     if isinstance(id_or_link, basestring) or is_dxlink(id_or_link):
         handler = get_handler(id_or_link)
         return handler.describe(**kwargs)
@@ -170,7 +213,19 @@ def describe(id_or_link, **kwargs):
                 else:
                     link = link['$dnanexus_link']['id']
             links.append(link)
-        data_object_descriptions = dxpy.api.system_describe_data_objects({'objects': links})
+
+        # Prepare input to system_describe_data_objects, the same fields will be passed
+        # for all data object classes; if a class doesn't include a field in its describe
+        # output, it will be ignored
+        describe_input = \
+            dict([(field, True) for field in kwargs['fields']]) if kwargs.get('fields', []) else True
+        describe_links_input = [{'id': link, 'describe': describe_input} for link in links]
+        bulk_describe_input = {'objects': describe_links_input}
+
+        if 'classDescribeOptions' in kwargs:
+            bulk_describe_input['classDescribeOptions'] = kwargs['classDescribeOptions']
+
+        data_object_descriptions = dxpy.api.system_describe_data_objects(bulk_describe_input)
         return [desc['describe'] for desc in data_object_descriptions['results']]
 
 def get_details(id_or_link, **kwargs):
