@@ -28,7 +28,7 @@ from __future__ import print_function, unicode_literals, division, absolute_impo
 
 import re
 import dxpy
-from ..utils import instance_type_to_sys_reqs
+from ..system_requirements import SystemRequirementsDict
 from ..bindings import DXDataObject, DXExecutable, DXAnalysis
 from ..exceptions import DXError
 from ..compat import basestring
@@ -189,7 +189,7 @@ class DXWorkflow(DXDataObject, DXExecutable):
         # A stage with the provided ID can't be found in the workflow, so look for it as a name
         stage_ids_matching_name = [stg['id'] for stg in self.stages if stg.get('name') == stage]
         if len(stage_ids_matching_name) == 0:
-            raise DXError('DXWorkflow: the given stage identifier could not be found as a stage ID nor as a stage name')
+            raise DXError('DXWorkflow: the given stage identifier ' + stage + ' could not be found as a stage ID nor as a stage name')
         elif len(stage_ids_matching_name) > 1:
             raise DXError('DXWorkflow: more than one workflow stage was found to have the name "' + stage + '"')
         else:
@@ -234,7 +234,7 @@ class DXWorkflow(DXDataObject, DXExecutable):
         if stage_input is not None:
             add_stage_input["input"] = stage_input
         if instance_type is not None:
-            add_stage_input["systemRequirements"] = instance_type_to_sys_reqs(instance_type)
+            add_stage_input["systemRequirements"] = SystemRequirementsDict.from_instance_type(instance_type).as_dict()
         self._add_edit_version_to_request(add_stage_input, edit_version)
         try:
             result = dxpy.api.workflow_add_stage(self._dxid, add_stage_input, **kwargs)
@@ -249,10 +249,10 @@ class DXWorkflow(DXDataObject, DXExecutable):
         :returns: Hash of stage descriptor in workflow
         '''
         stage_id = self._get_stage_id(stage)
-        try:
-            return next(stage for stage in self.stages if stage['id'] == stage_id)
-        except StopIteration:
+        result = next((stage for stage in self.stages if stage['id'] == stage_id), None)
+        if result is None:
             raise DXError('The stage ID ' + stage_id + ' could not be found')
+        return result
 
     def remove_stage(self, stage, edit_version=None, **kwargs):
         '''
@@ -427,7 +427,7 @@ class DXWorkflow(DXDataObject, DXExecutable):
         if stage_input:
             update_stage_input["input"] = stage_input
         if instance_type is not None:
-            update_stage_input["systemRequirements"] = instance_type_to_sys_reqs(instance_type)
+            update_stage_input["systemRequirements"] = SystemRequirementsDict.from_instance_type(instance_type).as_dict()
         if update_stage_input:
             update_input = {"stages": {stage_id: update_stage_input}}
             self._add_edit_version_to_request(update_input, edit_version)
@@ -471,7 +471,7 @@ class DXWorkflow(DXDataObject, DXExecutable):
             for stage, value in kwargs['stage_instance_types'].items():
                 if stage != '*':
                     stage = self._get_stage_id(stage)
-                run_input['stageSystemRequirements'][stage] = instance_type_to_sys_reqs(value)
+                run_input['stageSystemRequirements'][stage] = SystemRequirementsDict.from_instance_type(value).as_dict()
 
         if kwargs.get('stage_folders') is not None:
             run_input['stageFolders'] = {}
@@ -486,6 +486,15 @@ class DXWorkflow(DXDataObject, DXExecutable):
                 for _stage in kwargs['rerun_stages']
             ]
 
+        if kwargs.get('ignore_reuse', False):
+            run_input['ignoreReuse'] = ['*']
+
+        if kwargs.get('ignore_reuse_stages') is not None:
+            run_input['ignoreReuse'] = [
+                _stage if _stage == '*' else self._get_stage_id(_stage)
+                for _stage in kwargs['ignore_reuse_stages']
+            ]
+
         return run_input
 
     def _run_impl(self, run_input, **kwargs):
@@ -503,6 +512,8 @@ class DXWorkflow(DXDataObject, DXExecutable):
         :type stage_folders: dict
         :param rerun_stages: A list of stage IDs, names, indices, and/or the string "*" to indicate which stages should be run even if there are cached executions available
         :type rerun_stages: list of strings
+        :param ignore_reuse_stages: Stages of a workflow (IDs, names, or indices) or "*" for which job reuse should be disabled
+        :type ignore_reuse_stages: list
         :returns: Object handler of the newly created analysis
         :rtype: :class:`~dxpy.bindings.dxanalysis.DXAnalysis`
 
